#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <dir.h>


#include "pakfile.h"
#include "cards.h"
#include "players.h"
#include "dio.h"
#include "textgrph.h"


char
   *lev_name[10] = {
      "First Level:", "Second Level:", "Third Level:", "Fourth Level:",
      "Fifth Level:", "Sixth Level:", "Seventh Level:", "Eighth Level:",
      "Ninth Level:", "Tenth Level:"
   },
   *no_add_card_pak = "Unable to load card pak \'%s\'.\n",
   *add_series = "Adding Series: %s [%d cards]\n";

CARD_INFO
   *cards[10],
   *cards_tail[10];
int
   number_cards[10],
   num_common,
   num_uncommon,
   num_rare,
   num_very_rare,
   num_unique;
CARD_PAK_INFO
   *card_paks,
   *card_paks_tail;


int AddCard(unsigned id, char *name, unsigned set, char level, char rarity,
            char element, char *p)
{
   int
      i;
   CARD_INFO
      *nc;

   nc = (CARD_INFO *)calloc(1, sizeof(CARD_INFO));
   if(!nc)
      return 0;

   nc->next = NULL;
   strcpy(nc->name, name);
   nc->id = id;
   nc->set = set;
   nc->level = level;
   nc->rarity = rarity;
   nc->element = element;
   for(i = 0; i < 4; i++)
      nc->power[i] = p[i];

   if(cards[level - 1] == NULL)
   {
      cards[level - 1] = nc;
      nc->prev = NULL;
   } else
   {
      cards_tail[level - 1]->next = nc;
      nc->prev = cards_tail[level - 1];
   }

   cards_tail[level - 1] = nc;
   number_cards[level - 1]++;

   switch(rarity)
   {
      case 'C': num_common++; break;
      case 'U': num_uncommon++; break;
      case 'R': num_rare++; break;
      case '*': num_very_rare++; break;
      case '!': num_unique++; break;
   }
   return 1;
}


int LoadCardData(int num_cards, char *fname)
{
   int
      i,
      cnt,
      fp;
   unsigned
      id,
      set;
   char
      name[33],
      level,
      rarity,
      element,
      p[4];

   fp = Pak_fopen(fname);
   if(fp == -1)
      return 0;

   for(cnt = 0; cnt < num_cards; cnt++)
   {
      Pak_fread(name, 1, 32, fp);
      Pak_fread(&id, 2, 1, fp);
      Pak_fread(&set, 2, 1, fp);
      Pak_fread(&level, 1, 1, fp);
      Pak_fread(&rarity, 1, 1, fp);
      Pak_fread(&element, 1, 1, fp);
      for(i = 0; i < 4; i++)
         Pak_fread(&p[i], 1, 1, fp);

      rarity = toupper(rarity);
      element = toupper(element);
      if(!AddCard(id, name, set, level, rarity, element, p))
      {
         Pak_fclose(fp);
         return 0;
      }
   }

   Pak_fclose(fp);
   return 1;
}


int AddCardSeries(char *fname)
{
   int
      fp;
   char
      *cp,
      buffer[128];
   CARD_PAK_INFO
      *np;

   np = (CARD_PAK_INFO *)calloc(1, sizeof(CARD_PAK_INFO));
   if(!np)
      return 0;

   cp = strchr(fname, '.');
   if(cp)
      *cp = 0;
   strcat(fname, ".hed");

   strcpy(np->pakname, fname);
   np->next = NULL;

   if(card_paks == NULL)
      card_paks = np;
   else
      card_paks_tail->next = np;

   card_paks_tail = np;

   fname[8] = 0;
   np->set_id = atoi(&fname[4]);
   fname[8] = '.';

   fp = Pak_fopen(fname);
   if(fp == -1)
      return 0;

   Pak_fgets(buffer, 64, fp);
   cp = strchr(buffer, '\n');
   if(cp)
      *cp = 0;
   strcpy(np->series_name, buffer);

   Pak_fgets(buffer, 64, fp);
   np->num_cards = atoi(buffer);

   Pak_fclose(fp);
   printf(add_series, np->series_name, np->num_cards);

   cp = strchr(fname, '.');
   if(cp)
      *cp = 0;
   strcat(fname, ".dat");

   return LoadCardData(np->num_cards, fname);
}


void FreeSeriesList(void)
{
   CARD_PAK_INFO
      *tmp,
      *cur;

   cur = card_paks;

   while(cur)
   {
      tmp = cur->next;
      free(cur);
      cur = tmp;
   }

   card_paks = NULL;
   card_paks_tail = NULL;
}


void FreeCardList(void)
{
   int
      i;
   CARD_INFO
      *tmp,
      *cur;

   for(i = 0; i < 8; i++)
   {
      cur = cards[i];
      while(cur)
      {
         tmp = cur->next;
         free(cur);
         cur = tmp;
      }
      cards[i] = NULL;
      cards_tail[i] = NULL;
   }
}


void FreeCards(void)
{
   FreeSeriesList();
   FreeCardList();
}


void PrintTotalNumberCards(void)
{
   int
      i;

   printf("\nTotal Cards:\n");
   for(i = 0; i < 10; i++)
   {
      printf("   %-15s %d\n", lev_name[i], number_cards[i]);
   }
   printf("Common: %d  Uncommon: %d  Rare: %d  Very Rare: %d  Unique: %d\n",
      num_common, num_uncommon, num_rare, num_very_rare, num_unique);

   printf("\n\n");
}


/*
   Loads all the cards into the appropriate lists.
   Returns 1 on success or 0 on failure.
*/
int LoadCards(void)
{
   int
      num_card_paks;
   struct find_t
      f_list;

   atexit(FreeCards);

   /* Add all the card paks as pakfiles */
   num_card_paks = 0;
   if(!_dos_findfirst("card*.pak", FA_ARCH | FA_RDONLY, &f_list))
   {
      if(!AddPakFile(f_list.name))
      {
         printf(no_add_card_pak, f_list.name);
         return 0;
      }
      num_card_paks++;
      AddCardSeries(f_list.name);

      while(!_dos_findnext(&f_list))
      {
         if(!AddPakFile(f_list.name))
         {
            printf(no_add_card_pak, f_list.name);
            return 0;
         }
         num_card_paks++;
         AddCardSeries(f_list.name);
      }
   } else
   {
      printf("There are no cards to load!\n");
      return 0;
   }

   PrintTotalNumberCards();
   return 1;
}


void CreateStartingDeck(unsigned long *deck)
{
   CARD_INFO
      *cur;
   int
      i,
      rand_num,
      cnt;


   /* Get first level cards */
   for(cnt = 0; cnt < 5; cnt++)
   {
      rand_num = rand() % number_cards[0];

      for(cur = cards[0], i = 0; i < rand_num; i++)
         cur = cur->next;

      deck[cnt] = (cur->id) + (1L << 16L);
   }

   /* Get Second level cards */
   for(cnt = 0; cnt < 2; cnt++)
   {
      rand_num = rand() % number_cards[1];

      for(cur = cards[1], i = 0; i < rand_num; i++)
         cur = cur->next;

      deck[cnt + 5] = cur->id + (2L << 16L);
   }

   /* Get Third level card */
   rand_num = rand() % number_cards[2];

   for(cur = cards[2], i = 0; i < rand_num; i++)
      cur = cur->next;

   deck[7] = cur->id + (3L << 16L);
}


void FillInCardInfo(CARD_INFO *c, unsigned short id, unsigned short level)
{
   CARD_INFO
      *cur;

   cur = cards[level];
   while(cur)
   {
      if(cur->id == id)
      {
         memcpy(c, cur, sizeof(CARD_INFO));
         c->next = NULL;
         c->prev = NULL;
         return;
      }
      cur = cur->next;
   }
}


void CreateBattleDeck(PLAYER_INFO *p, CARD_INFO *pcards)
{
   int
      i,
      c,
      already_picked,
      rv,
      picked[5];

   for(i = 0; i < 5; i++)
      picked[i] = -1;

   i = 0;
   do
   {
      already_picked = 0;
      rv = rand() % p->number_of_cards;
      for(c = 0; c < i; c++)
      {
         if(rv == picked[c])
         {
            already_picked = 1;
            break;
         }
      }

      if(!already_picked)
      {
         picked[i] = rv;
         FillInCardInfo(&pcards[i], (p->deck[rv] & 0xFFFF),
            (p->deck[rv] >> 16L) - 1);
         i++;
      }
   } while(i < 5);
}


void CreateComputerBattleDeck(CARD_INFO *c_cards, CARD_INFO *opponents)
{
   int
      i,
      rv;
   CARD_INFO
      *cur;

   for(i = 0; i < 5; i++)
   {
      rv = rand() % number_cards[opponents[i].level - 1];
      cur = cards[opponents[i].level - 1];

      while(rv)
      {
         rv--;
         cur = cur->next;
      }

      memcpy(&c_cards[i], cur, sizeof(CARD_INFO));
      c_cards[i].next = NULL;
      c_cards[i].prev = NULL;
   }
}


void DrawCard(int x, int y, int color, CARD_INFO *c)
{
   int
      i,
      column,
      line;
   char
      val,
      *cp,
      buf[33];

   Box(x, y, x + 14, y + 7, SINGLE_BOX, 1, 15, color);

   /* Display Cards power */
   Cursor(x + 7, y + 1);
   val = (c->power[0] < 10) ? (c->power[0] + '0') : 'A';
   WriteChar(val, 15, color);

   Cursor(x + 13, y + 4);
   val = (c->power[1] < 10) ? (c->power[1] + '0') : 'A';
   WriteChar(val, 15, color);

   Cursor(x + 7, y + 6);
   val = (c->power[2] < 10) ? (c->power[2] + '0') : 'A';
   WriteChar(val, 15, color);

   Cursor(x + 1, y + 4);
   val = (c->power[3] < 10) ? (c->power[3] + '0') : 'A';
   WriteChar(val, 15, color);

   /* Display cards rarity */
   Cursor(x + 1, y + 1);
   WriteChar('(', 15, color);
   Cursor(x + 2, y + 1);
   WriteChar(c->rarity, 15, color);
   Cursor(x + 3, y + 1);
   WriteChar(')', 15, color);

   /* Display cards element */
   Cursor(x + 11, y + 1);
   WriteChar('(', 15, color);
   Cursor(x + 12, y + 1);
   WriteChar(c->element, 15, color);
   Cursor(x + 13, y + 1);
   WriteChar(')', 15, color);

   sprintf(buf, "(%d)", c->level);
   for(i = 0; buf[i]; i++)
   {
      Cursor(x + 1 + i, y + 6);
      WriteChar(buf[i], 15, color);
   }

   /* Display cards name */
   strcpy(buf, c->name);

   line = 2 + y;
   cp = strtok(buf, " ");
   while(cp)
   {
      column = x + 7 - (strlen(cp) / 2);
      WriteString(cp, 15, color, column, line);
      line++;
      cp = strtok(NULL, " ");
   }
}


char *GetSeriesName(unsigned short set_id)
{
   CARD_PAK_INFO
      *cur;
   static char
      sn_buffer[64];

   cur = card_paks;
   while(cur)
   {
      if(cur->set_id == set_id)
      {
         strcpy(sn_buffer, cur->series_name);
         return sn_buffer;
      }
      cur = cur->next;
   }

   strcpy(sn_buffer, "Unknown");
   return sn_buffer;
}

char *GetCardRarityName(int rarity)
{
   switch(rarity)
   {
      case 'C': return "Common";
      case 'U': return "Uncommon";
      case 'R': return "Rare";
      case '*': return "Very Rare";
      case '!': return "Unique";
   }
   return "Unknown";
}


char *GetCardElementName(int element)
{
   switch(element)
   {
      case 'A': return "Air";
      case 'E': return "Earth";
      case 'F': return "Fire";
      case 'W': return "Water";
      case 'L': return "Light";
      case 'D': return "Dark";
   }
   return "Unknown";
}


void DisplayCardData(int level, int id)
{
   CARD_INFO
      *cur;
   char
      buffer[80];

   cur = cards[level];
   while(cur)
   {
      if(cur->id == id)
      {
         DrawCard(50, 10, 1, cur);

         sprintf(buffer, "Card Level: %d", cur->level);
         WriteString(buffer, 14, 1, 3, 6);

         sprintf(buffer, "Name:     %s", cur->name);
         WriteString(buffer, 14, 1, 3, 7);

         sprintf(buffer, "Card ID:  %u", cur->id);
         WriteString(buffer, 14, 1, 3, 8);

         sprintf(buffer, "Set:      %s", GetSeriesName(cur->set));
         WriteString(buffer, 14, 1, 3, 9);

         sprintf(buffer, "Rarity:   %s", GetCardRarityName(cur->rarity));
         WriteString(buffer, 14, 1, 3, 10);

         sprintf(buffer, "Element:  %s", GetCardElementName(cur->element));
         WriteString(buffer, 14, 1, 3, 11);
         return;
      }
      cur = cur->next;
   }
}


void DisplayCardInfoPageBackground(char *title, PLAYER_INFO *p)
{
   char
      buffer[64];

   Grid(0, 0, 79, 24, 3, 1, 2);

   /* Draw Title */
   Cursor(0, 0);
   RepeatChar(' ', 80, 14, 1);
   WriteString(title, 14, 1, 40 - strlen(title) / 2, 0);

   Grid(4, 5, 42, 15, 0, 0, 0);
   Grid(52, 11, 66, 18, 0, 0, 0);

   if(p)
   {
      Grid(4, 19, 42, 23, 0, 0, 0);
      Box(2, 18, 40, 22, SINGLE_BOX, 1, 14, 1);

      sprintf(buffer, "Game Points: %ld", p->game_points);
      WriteString(buffer, 14, 1, 4, 19);

      sprintf(buffer, "Wins:   %u", p->wins);
      WriteString(buffer, 14, 1, 4, 20);

      sprintf(buffer, "Losses: %u", p->losses);
      WriteString(buffer, 14, 1, 4, 21);
   }
}


void DisplayCardInfoPage(void)
{
   Box(2, 4, 40, 14, SINGLE_BOX, 1, 14, 1);
}


void ViewAllCards(void)
{
   CARD_INFO
      *cur_card_info;
   int
      done,
      cur_card,
      k,
      redraw,
      level_num;
   char
      buffer[80];

   CursorOff();

   DisplayCardInfoPageBackground(" View All Cards", 0);

   cur_card_info = cards[0];
   level_num = 0;
   cur_card = 1;
   done = 0;
   redraw = 1;
   while(!done)
   {
      if(redraw)
      {
         DisplayCardInfoPage();

         if(number_cards[level_num] == 0)
         {
            sprintf(buffer, "There are no level #%d cards.\n", level_num + 1);
         } else
         {
            sprintf(buffer, "Card #%d of %d", cur_card, number_cards[level_num]);

            DisplayCardData(level_num, cur_card_info->id);
         }
         WriteString(buffer, 14, 1, 3, 5);
         redraw = 0;
      }


      fflush(stdin);
      k = dio_getchar();
      fflush(stdin);

      if(k == ESCAPE)
      {
         done = 1;
      } else if(k == UP)
      {
         if(number_cards[level_num])
         {
            if(cur_card_info->prev)
            {
               cur_card_info = cur_card_info->prev;
               cur_card--;
               redraw = 1;
            }
         }
      } else if(k == DOWN)
      {
         if(number_cards[level_num])
         {
            if(cur_card_info->next)
            {
               cur_card_info = cur_card_info->next;
               cur_card++;
               redraw = 1;
            }
         }
      } else if(k == RIGHT)
      {
         if(level_num < 9)
         {
            level_num++;
            cur_card_info = cards[level_num];
            cur_card = 1;
            redraw = 1;
         }
      } else if(k == LEFT)
      {
         if(level_num)
         {
            level_num--;
            cur_card_info = cards[level_num];
            cur_card = 1;
            redraw = 1;
         }
      }
   }

   CursorOn();
   dio_clear();
}


long CalculateCardCost(int level, int rarity)
{
   long
      rarity_cost = 0;

   switch(rarity)
   {
      case 'C': rarity_cost = 2; break;
      case 'U': rarity_cost = 3; break;
      case 'R': rarity_cost = 4; break;
      case '*': rarity_cost = 5; break;
      case '!': rarity_cost = 6; break;
   }

   return (level * rarity_cost + 3L);
}


int GetCardRarity(int level, unsigned short id)
{
   CARD_INFO
      *cur;

   cur = cards[level];
   while(cur)
   {
      if(cur->id == id)
         return cur->rarity;
      cur = cur->next;
   }
   return 'C';
}


void SellAtShop(void)
{
   int
      done,
      redraw,
      cur_card,
      k;
   unsigned short
      id,
      level_num;
   long
      cost;
   char
      buffer[80];

   CursorOff();

   if(player[0].number_of_cards == 0)
   {
      dio_clear();
      MessageBox("Sorry you don't have any cards to sell.");
      CursorOn();
      dio_clear();
      return;
   }

   DisplayCardInfoPageBackground("Sell Cards", &player[0]);

   cur_card = 0;
   done = 0;
   redraw = 1;
   while(!done)
   {
      if(redraw)
      {
         DisplayCardInfoPage();
         sprintf(buffer, "Card #%d of %d", cur_card + 1,
            player[0].number_of_cards);
         WriteString(buffer, 14, 1, 3, 5);

         level_num = (unsigned short)(player[0].deck[cur_card] >> 16L) - 1;
         id = (unsigned short)(player[0].deck[cur_card] & 0xFFFF);

         cost = CalculateCardCost(level_num, GetCardRarity(level_num, id)) / 2;

         sprintf(buffer, "Sell Price: %ld", cost);
         WriteString(buffer, 14, 1, 3, 12);

         DisplayCardData(level_num, id);
         redraw = 0;
      }

      fflush(stdin);
      k = dio_getchar();
      fflush(stdin);

      if(k == ESCAPE)
      {
         done = 1;
      } else if(k == UP)
      {
         if(cur_card)
         {
            redraw = 1;
            cur_card--;
         }
      } else if(k == DOWN)
      {
         if(cur_card < (player[0].number_of_cards - 1))
         {
            redraw = 1;
            cur_card++;
         }
      } else if(k == ENTER)
      {
         WriteString("Are you sure? (y/n)", 14, 1, 3, 13);

         do
         {
            fflush(stdin);
            k = dio_getchar();
            fflush(stdin);
         } while(k != 'y' && k != 'Y' && k != 'n' && k != 'N');

         redraw = 1;
         if(k == 'y' || k == 'Y')
         {
            MessageBox("Card sold!");
            DisplayCardInfoPageBackground("Sell Cards", &player[0]);

            if(player[0].number_of_cards == 1)
            {
               memset(player[0].deck, 0, 4 * 500);
               done = 1;
            } else
            {
               if(cur_card != (player[0].number_of_cards - 1))
               {
                  memcpy(&player[0].deck[cur_card], &player[0].deck[cur_card+1],
                     (player[0].number_of_cards - cur_card) * 4);
               } else
               {
                  cur_card--;
               }
            }

            player[0].number_of_cards--;

            if(player[0].number_of_cards == 0)
               done = 1;

            player[0].game_points += cost;
         }
      }
   }

   dio_clear();
   CursorOn();
   SavePlayer(&player[0]);
}


typedef struct
{
   char
      name[32];
   unsigned long
      level,
      id;
   long
      cost;
} SHOP_CARD_INFO;


void BuyAtShop(void)
{
   CARD_INFO
      *cur;
   SHOP_CARD_INFO
      shop_cards[10];
   int
      lev,
      i,
      rv,
      k,
      fg,
      bg,
      redraw,
      choice,
      done;
   char
      buffer[80];

   /* Generate cards for sale */
   for(i = 0; i < 10; i++)
   {
      lev = rand() % 10;

      cur = cards[lev];
      rv = rand() % number_cards[lev];

      for(k = 0; k < rv; k++)
         cur = cur->next;

      strcpy(shop_cards[i].name, cur->name);
      shop_cards[i].level = lev;
      shop_cards[i].id = cur->id;
      shop_cards[i].cost = CalculateCardCost(lev + 1, cur->rarity);
   }

   dio_clear();
   CursorOff();

   Grid(0, 0, 79, 24, 3, 1, 2);
   Cursor(0, 0);
   RepeatChar(' ', 80, 14, 1);

   WriteString("BUY", 14, 1, 38, 0);

   Grid(16, 5, 66, 18, 0, 0, 0);
   Box(14, 4, 64, 17, SINGLE_BOX, 1, 14, 1);

   sprintf(buffer, "%-32s   Level   Cost", "Card Name");
   WriteString(buffer, 14, 1, 16, 5);

   /* Do Shop stuff */
   done = 0;
   choice = 0;
   redraw = 1;
   while(!done)
   {
      if(redraw)
      {
         for(i = 0; i < 11; i++)
         {
            if(i < 10)
            {
               sprintf(buffer, " %-32s  %3ld    %4ld ", shop_cards[i].name,
                  shop_cards[i].level + 1, shop_cards[i].cost);
            } else
            {
               sprintf(buffer, " %-32s              ", "Go Back");
            }

            if(i < 10 && player[0].game_points < shop_cards[i].cost)
            {
               if(choice == i)
                  fg = RED;
               else
                  fg = LIGHTRED;
            } else
            {
               if(choice == i)
                  fg = BLACK;
               else
                  fg = WHITE;
            }

            if(choice == i)
               bg = CYAN;
            else
               bg = BLUE;

            WriteString(buffer, fg, bg, 16, i + 6);
         }

         /* Display point box */
         Grid(16, 21, 66, 23, 0, 0, 0);
         Box(14, 20, 64, 22, SINGLE_BOX, 1, 14, 1);

         sprintf(buffer, "You have %ld points.", player[0].game_points);
         WriteString(buffer, 14, 1, 16, 21);

         redraw = 0;
      }

      fflush(stdin);
      k = dio_getchar();
      fflush(stdin);

      if(k == ESCAPE)
      {
         done = 1;
      } else if(k == UP)
      {
         if(choice)
         {
            choice--;
            redraw = 1;
         }
      } else if(k == DOWN)
      {
         if(choice < 10)
         {
            choice++;
            redraw = 1;
         }
      } else if(k == ENTER)
      {
         if(choice < 10)
         {
            if(player[0].game_points >= shop_cards[choice].cost)
            {
               if(player[0].number_of_cards < 500)
               {
                  player[0].deck[player[0].number_of_cards] =
                     shop_cards[choice].id + ((shop_cards[choice].level + 1) << 16L);
                  player[0].number_of_cards++;
                  player[0].game_points -= shop_cards[choice].cost;

                  MessageBox("You bought \'%s\'!", shop_cards[choice].name);
                  redraw = 1;
               }
            }
         } else
         {
            done = 1;
         }
      }
   }

   dio_clear();
   CursorOn();
   SavePlayer(&player[0]);
}


void Shop(void)
{
   static char
      *shop_menu[] = {
         "Buy",
         "Sell",
         "Go Back"
      };
   int
      done,
      k;

   done = 0;
   while(!done)
   {
      k = Menu("SHOP MENU", shop_menu, 3);

      switch(k)
      {
         case 0:
            BuyAtShop();
            break;
         case 1:
            SellAtShop();
            break;
         case 2:
            done = 1;
      }
   }
   dio_clear();
}


void ViewPlayersCards(PLAYER_INFO *p)
{
   int
      done,
      cur_card,
      redraw,
      k;
   unsigned short
      id,
      level_num;
   char
      buffer[80];

   CursorOff();
   sprintf(buffer, "View %s's Cards", p->name);
   DisplayCardInfoPageBackground(buffer, p);

   cur_card = 0;
   done = 0;
   redraw = 1;
   while(!done)
   {
      if(redraw)
      {
         DisplayCardInfoPage();

         sprintf(buffer, "Card #%d of %d", cur_card + 1, p->number_of_cards);
         WriteString(buffer, 14, 1, 3, 5);

         level_num = (unsigned short)(p->deck[cur_card] >> 16L) - 1;
         id = (unsigned short)(p->deck[cur_card] & 0xFFFF);

         DisplayCardData(level_num, id);
         redraw = 0;
      }

      fflush(stdin);
      k = dio_getchar();
      fflush(stdin);

      if(k == ESCAPE)
      {
         done = 1;
      } else if(k == UP)
      {
         if(cur_card)
         {
            cur_card--;
            redraw = 1;
         }
      } else if(k == DOWN)
      {
         if(cur_card < (p->number_of_cards - 1))
         {
            cur_card++;
            redraw = 1;
         }
      }
   }

   CursorOn();
   dio_clear();
}


/*
   Returns the menu item picked (1 to num_items)
*/
int Menu(char *menu_name, char **menu_items, int num_items)
{
   int
      redraw,
      done,
      i,
      k,
      fg,
      bg,
      xp,
      yp,
      len,
      cur_spot,
      max_len;
   char
      buffer[80];

   max_len = 0;
   for(i = 0; i < num_items; i++)
   {
      if(max_len < strlen(menu_items[i]))
         max_len = strlen(menu_items[i]);
   }

   /* Draw the screen */
   CursorOff();
   dio_clear();
   Grid(0, 0, 79, 24, 3, 1, 2);

   /* Draw Title */
   Cursor(0, 0);
   RepeatChar(' ', 80, 14, 1);
   WriteString(menu_name, 14, 1, 40 - strlen(menu_name) / 2, 0);

   xp = 40 - (max_len / 2) - 5;
   yp = 12 - (num_items / 2) - 2;

   Grid(xp + 2, yp + 1, xp + max_len + 10, yp + num_items + 2, 0, 0, 0);
   Box(xp, yp, xp + max_len + 8, yp + num_items + 1, SINGLE_BOX, 1, 14, 1);

   done = 0;
   redraw = 1;
   cur_spot = 0;
   while(!done)
   {
      if(redraw)
      {
         /* Draw Menu Choices */
         for(i = 0; i < num_items; i++)
         {
            sprintf(buffer, " %d. %s ", i + 1, menu_items[i]);
            fg = 14;
            bg = 1;
            if(cur_spot == i)
            {
               fg = 0;
               bg = 3;
            }
            WriteString(buffer, fg, bg, xp + 2, yp + 1 + i);
            len = strlen(buffer);
            if(len < (max_len + 6))
               RepeatChar(' ', 6 + max_len - len, fg, bg);
         }
         redraw = 0;
      }

      fflush(stdin);
      k = dio_getchar();
      fflush(stdin);

      if(k == UP)
      {
         if(cur_spot)
         {
            cur_spot--;
            redraw = 1;
         }
      } else if(k == DOWN)
      {
         if(cur_spot < (num_items - 1))
         {
            cur_spot++;
            redraw = 1;
         }
      } else if(k == ENTER)
      {
         done = 1;
      }
   }

   CursorOn();
   return cur_spot;
}


void MessageBox(char *text, ...)
{
   va_list
      args;
   char
      buffer[80];
   int
      xp,
      xadd,
      len;

   va_start(args, text);
   vsprintf(buffer, text, args);
   va_end(args);

   len = strlen(buffer);

   xp = 38 - (len / 2);
   xadd = len + 4;

   Grid(xp + 2, 12, xp + xadd + 2, 14, BLACK, BLACK, 0);
   Box(xp, 11, xp + xadd, 13, SINGLE_BOX, 1, 15, 7);

   WriteString(buffer, 0, 7, xp + 2, 12);

   fflush(stdin);
   dio_getchar();
   fflush(stdin);
}