/*
    DIO.H - Direct Input/Output routines.

*/
#ifndef _DIO_H
   #define _DIO_H

   #ifdef _cplusplus
      extern "C" {
   #endif /* _cplusplus */

   #define _BORLAND_C     1
   #define _AIX_C         2
   #define _DJGPP_C       3   /* shares a lot in common with _BORLAND_C */

   /* define PLATFORM to be one of the above compilers */
   #define PLATFORM _BORLAND_C

   /* define some colors. */
#if (PLATFORM == _AIX_C)
   #define BLACK           0
   #define BLUE            1
   #define GREEN           2
   #define CYAN            3
   #define RED             4
   #define MAGENTA         5
   #define BROWN           6
   #define LIGHT_GRAY      7
   #define DARK_GRAY       8
   #define LIGHTBLUE       9
   #define LIGHTGREEN     10
   #define LIGHTCYAN      11
   #define LIGHTRED       12
   #define LIGHTMAGENTA   13
   #define YELLOW         14
   #define WHITE          15
#endif

   /* define some colors */
   #if (PLATFORM == _BORLAND_C) || (PLATFORM == _DJGPP_C)

      #define BACKSPACE     8
      #define TAB           9
      #define ENTER        13
      #define ESCAPE       27

      /* Following keys are the scan code + 1000 */
      #define F1         1059
      #define F2         1060
      #define F3         1061
      #define F4         1062
      #define F5         1063
      #define F6         1064
      #define F7         1065
      #define F8         1066
      #define F9         1067
      #define F10        1068
   
      #define LEFT       1075
      #define RIGHT      1077
      #define UP         1072
      #define DOWN       1080
   
      #define HOME       1071
      #define END        1079
      #define INSERT     1082
      #define DELETE     1083
      #define PGUP       1073
      #define PGDN       1081
   
   #elif (PLATFORM == _AIX_C)

      /* TODO: Fill in values for keys */

   #endif   /* PLATFORM == ... */

   /* Macro to determine if a character is displayable or not */
   #define PRINTABLE_CHARACTER(x)   ((x) >= ' ' && (x) <= '~')

   void dio_on(void);
   void dio_off(void);
   void dio_setup_exit_keys(int *keys, int num_keys);
   int dio_rows(void);
   int dio_cols(void);
   void dio_clear(void);
   void dio_update(void);
   int dio_getchar(void);
   void dio_moveto(int row, int column);
   void dio_putchar(int c);
   void dio_puts(const char *s);
   void dio_display(const char *s, int row, int column, int length);
   int dio_edit(char *s, int row, int col, int flen, int slen);
   void dio_padchar(char c);
   void dio_textcolor(int color);
   void dio_textbackground(int color);
   void dio_sound(int state);
   void dio_backspace_exit(int state);

   #ifdef _cplusplus
      }
   #endif /* _cplusplus */
   
   
#endif /* _DIO_H */
