#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "pakfile.h"
#include "cards.h"
#include "players.h"
#include "dio.h"
#include "ai.h"


void MainMenu(void);
void PlayMenu(void);
void PickFieldType(void);
void PlayOnSameComputer(void);

int
   play_against_computer,
   game_type,
   num_players,
   win_points;
int
   edit_exit_keys[] = { ENTER, ESCAPE };
CARD_INFO
   field_cards[3][3];
int
   field_color[3][3],
   field_element[3][3],
   field_element_bonus[3][3];



int main(void)
{
   exit_game = 0;
   srand(time(0));
   dio_on();
   dio_setup_exit_keys(edit_exit_keys, 2);

   InitPakFiles();
   LoadPlayerNames();

   if(!LoadCards())
      return 1;

   dio_puts("Press any key to continue...");
   fflush(stdin);
   dio_getchar();
   fflush(stdin);

   dio_clear();

   game_state = STATE_PLAYER_MENU;

   while(!exit_game)
   {
      switch(game_state)
      {
         case STATE_PLAYER_MENU:
            PlayerMenu();
            break;
         case STATE_GAME_MENU:
            MainMenu();
            break;
         default:
            printf("Error: Invalid game state (%d)!\n", game_state);
      }
   }

   dio_off();
   return 0;
}


void MainMenu(void)
{
   static char
      *main_menu[] = {
         "Play",
         "View Your Cards",
         "View All Cards",
         "Trade Cards",
         "Visit Store",
         "Select New Player",
         "Quit"
      };
   int
      done,
      k,
      choice;

   done = 0;
   while(!done)
   {
      choice = Menu("MAIN MENU", main_menu, 7);

      switch(choice)
      {
         case 0:
            if(player[0].number_of_cards >= 5)
               PlayMenu();
            break;
         case 1:
            ViewPlayersCards(&player[0]);
            break;
         case 2:
            ViewAllCards();
            break;
         case 3:
            SameComputerTrade();
            break;
         case 4:
            Shop();
            break;
         case 5:
            dio_clear();
            if(!SavePlayer(&player[0]))
            {
               dio_display("There was an error saving your character.",
                  12, 0, 41);
               dio_display("Really select new character? (y/n)", 13, 0, 34);

               do
               {
                  fflush(stdin);
                  k = dio_getchar();
                  fflush(stdin);
               } while(k != 'y' && k != 'Y' && k != 'n' && k != 'N');

               if(k == 'y' || k == 'Y')
               {
                  memset(&player[0], 0, sizeof(PLAYER_INFO));
                  game_state = STATE_PLAYER_MENU;
                  done = 1;
               } else
               {
                  dio_clear();
               }
            } else
            {
               memset(&player[0], 0, sizeof(PLAYER_INFO));
               game_state = STATE_PLAYER_MENU;
               done = 1;
            }
            break;
         case 6:
            dio_clear();
            exit_game = 1;
            done = 1;
            break;
      }
   }
}


void PlayMenu(void)
{
   static char
      *play_menu[] = {
         "Person vs Computer",
         "Person vs Person - Same Computer",
//         "Person vs Person - Serial Link",
//         "Person vs Person - IPX Network",
         "Go Back"
      };
   int
      done,
      k,
      choice;

   dio_clear();
   done = 0;
   while(!done)
   {
      choice = Menu("PLAY MENU", play_menu, 3);

      switch(choice)
      {
         case 0:
            play_against_computer = 1;
            PickFieldType();
            PlayOnSameComputer();
            done = 1;
            break;
         case 1:
            play_against_computer = 0;
            if(!SelectPlayer(player[0].name, &player[1]))
            {
               done = 1;
            } else
            {
               if(player[1].number_of_cards >= 5)
               {
                  PickFieldType();
                  PlayOnSameComputer();
               }
               done = 1;
            }
            break;
//         case 2:
//            break;
//         case 3:
//            break;
         case 2:
            done = 1;
            break;
      }
   }
   dio_clear();
}



void PickFieldType(void)
{
   static char
      *field_type[] = {
         "Normal",
         "Elemental",
         "Chain",
         "Chain Elemental"
      };
   int
      x,
      y,
      e,
      done,
      k,
      choice;

   memset(field_cards, 0, sizeof(CARD_INFO) * 9);
   memset(field_element, 0, sizeof(int) * 9);
   memset(field_color, 0, sizeof(int) * 9);
   memset(field_element_bonus, 0, sizeof(int) * 9);


   dio_clear();
   done = 0;
   while(!done)
   {
      choice = Menu("FIELD MENU", field_type, 4);

      switch(choice)
      {
         case 0:
            game_type = 0;
            win_points = 1;
            done = 1;
            break;
         case 1:
            game_type = 1;
            win_points = 2;
            done = 1;
            for(y = 0; y < 3; y++)
            {
               for(x = 0; x < 3; x++)
               {
                  switch(rand() % 6)
                  {
                     case 0: e = 'A'; break;
                     case 1: e = 'W'; break;
                     case 2: e = 'F'; break;
                     case 3: e = 'E'; break;
                     case 4: e = 'L'; break;
                     case 5: e = 'D'; break;
                  }
                  field_element[y][x] = e;
               }
            }
            break;
         case 2:
            game_type = 2;
            win_points = 3;
            done = 1;
            break;
         case 3:
            game_type = 3;
            win_points = 4;
            done = 1;
            for(y = 0; y < 3; y++)
            {
               for(x = 0; x < 3; x++)
               {
                  switch(rand() % 6)
                  {
                     case 0: e = 'A'; break;
                     case 1: e = 'W'; break;
                     case 2: e = 'F'; break;
                     case 3: e = 'E'; break;
                     case 4: e = 'L'; break;
                     case 5: e = 'D'; break;
                  }
                  field_element[y][x] = e;
               }
            }
            break;
      }
   }

   dio_clear();
}


int GetFieldElementBonus(CARD_INFO *c, int cur_x, int cur_y)
{
   switch(c->element)
   {
      case 'A':
         if(field_element[cur_y][cur_x] == 'A')
            return 1;
         else if(field_element[cur_y][cur_x] == 'E')
            return -1;
         break;
      case 'F':
         if(field_element[cur_y][cur_x] == 'F')
            return 1;
         else if(field_element[cur_y][cur_x] == 'W')
            return -1;
         break;
      case 'E':
         if(field_element[cur_y][cur_x] == 'E')
            return 1;
         else if(field_element[cur_y][cur_x] == 'F')
            return -1;
         break;
      case 'W':
         if(field_element[cur_y][cur_x] == 'W')
            return 1;
         else if(field_element[cur_y][cur_x] == 'A')
            return -1;
         break;
      case 'L':
         if(field_element[cur_y][cur_x] == 'L')
            return 1;
         else if(field_element[cur_y][cur_x] == 'D')
            return -1;
         break;
      case 'D':
         if(field_element[cur_y][cur_x] == 'D')
            return 1;
         else if(field_element[cur_y][cur_x] == 'L')
            return -1;
         break;
   }
   return 0;
}


void SetupElementBonus(int cur_x, int cur_y)
{
   field_element_bonus[cur_y][cur_x] =
      GetFieldElementBonus(&field_cards[cur_y][cur_x], cur_x, cur_y);
}


void NormalFlip(int flip_color, int cur_x, int cur_y)
{
   CARD_INFO
      *c1,
      *c2;
   int
      p1,
      p2;

   c1 = &field_cards[cur_y][cur_x];

   /* Check Top */
   if(cur_y)
   {
      c2 = &field_cards[cur_y - 1][cur_x];
      if(c2->name[0])
      {
         p1 = c1->power[TOP_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[BOTTOM_SIDE] + field_element_bonus[cur_y - 1][cur_x];
         if(p1 > p2)
            field_color[cur_y - 1][cur_x] = flip_color;
      }
   }

   /* Check right */
   if(cur_x < 2)
   {
      c2 = &field_cards[cur_y][cur_x + 1];
      if(c2->name[0])
      {
         p1 = c1->power[RIGHT_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[LEFT_SIDE] + field_element_bonus[cur_y][cur_x + 1];
         if(p1 > p2)
            field_color[cur_y][cur_x + 1] = flip_color;
      }
   }

   /* Check Bottom */
   if(cur_y < 2)
   {
      c2 = &field_cards[cur_y + 1][cur_x];
      if(c2->name[0])
      {
         p1 = c1->power[BOTTOM_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[TOP_SIDE] + field_element_bonus[cur_y + 1][cur_x];
         if(p1 > p2)
            field_color[cur_y + 1][cur_x] = flip_color;
      }
   }

   /* Check Left Side */
   if(cur_x)
   {
      c2 = &field_cards[cur_y][cur_x - 1];
      if(c2->name[0])
      {
         p1 = c1->power[LEFT_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[RIGHT_SIDE] + field_element_bonus[cur_y][cur_x - 1];
         if(p1 > p2)
            field_color[cur_y][cur_x - 1] = flip_color;
      }
   }
}


void ChainFlip(int flip_color, int cur_x, int cur_y)
{
   CARD_INFO
      *c1,
      *c2;
   int
      p1,
      p2;

   c1 = &field_cards[cur_y][cur_x];

   /* Check Top */
   if(cur_y)
   {
      c2 = &field_cards[cur_y - 1][cur_x];
      if(c2->name[0] && field_color[cur_y - 1][cur_x] != flip_color)
      {
         p1 = c1->power[TOP_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[BOTTOM_SIDE] + field_element_bonus[cur_y - 1][cur_x];
         if(p1 > p2)
         {
            field_color[cur_y - 1][cur_x] = flip_color;
            ChainFlip(flip_color, cur_x, cur_y - 1);
         }
      }
   }

   /* Check right */
   if(cur_x < 2)
   {
      c2 = &field_cards[cur_y][cur_x + 1];
      if(c2->name[0] && field_color[cur_y][cur_x + 1] != flip_color)
      {
         p1 = c1->power[RIGHT_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[LEFT_SIDE] + field_element_bonus[cur_y][cur_x + 1];
         if(p1 > p2)
         {
            field_color[cur_y][cur_x + 1] = flip_color;
            ChainFlip(flip_color, cur_x + 1, cur_y);
         }
      }
   }

   /* Check Bottom */
   if(cur_y < 2)
   {
      c2 = &field_cards[cur_y + 1][cur_x];
      if(c2->name[0] && field_color[cur_y + 1][cur_x] != flip_color)
      {
         p1 = c1->power[BOTTOM_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[TOP_SIDE] + field_element_bonus[cur_y + 1][cur_x];
         if(p1 > p2)
         {
            field_color[cur_y + 1][cur_x] = flip_color;
            ChainFlip(flip_color, cur_x, cur_y + 1);
         }
      }
   }

   /* Check Left Side */
   if(cur_x)
   {
      c2 = &field_cards[cur_y][cur_x - 1];
      if(c2->name[0]  && field_color[cur_y][cur_x - 1] != flip_color)
      {
         p1 = c1->power[LEFT_SIDE] + field_element_bonus[cur_y][cur_x];
         p2 = c2->power[RIGHT_SIDE] + field_element_bonus[cur_y][cur_x - 1];
         if(p1 > p2)
         {
            field_color[cur_y][cur_x - 1] = flip_color;
            ChainFlip(flip_color, cur_x - 1, cur_y);
         }
      }
   }
}



/*
   Return 1 if player1 won, returns 2 if player2 won, otherwise returns 0
*/
int CheckForWin(void)
{
   int
      x,
      y,
      red = 0,
      blue = 0,
      total = 0;

   for(y = 0; y < 3; y++)
   {
      for(x = 0; x < 3; x++)
      {
         if(field_cards[y][x].name[0])
         {
            total++;
            if(field_color[y][x] == 1)
               blue++;
            else
               red++;
         }
      }
   }

   if(total == 9)
   {
      if(blue > red)
         return 1;
      return 2;
   }
   return 0;
}


void DrawField(void)
{
   int
      i,
      c,
      x,
      y,
      xp,
      yp;
   char
      buffer[5];

   /* Draw the field */
   for(y = 0; y < 3; y++)
   {
      for(x = 0; x < 3; x++)
      {
         xp = 18 + (x * 15);
         yp = y * 8;

         if(field_cards[y][x].name[0])
         {
            DrawCard(xp, yp, field_color[y][x], &field_cards[y][x]);
            if(game_type == 1 || game_type == 3) /* element */
            {
               sprintf(buffer, "(%+d)", field_element_bonus[y][x]);
               c = 14 - strlen(buffer) + xp;
               for(i = 0; buffer[i]; i++)
               {
                  Cursor(c + i, yp + 6);
                  WriteChar(buffer[i], 15, field_color[y][x]);
               }
            }
         } else
         {
            Grid(xp, yp, xp + 14, yp + 7, 15, 2, 0);

            if(game_type == 1 || game_type == 3) /* Element */
            {
               Cursor(xp + 7, yp + 4);
               WriteChar(field_element[y][x], 15, 2);
            }
         }
      }
   }
}


void DrawBattleDeck(CARD_INFO *deck, int num_cards, int x, int color)
{
   int
      i;

   for(i = 0; i < num_cards; i++)
      DrawCard(x, (i * 3), color, &deck[i]);
}


void PlayOnSameComputer(void)
{
   int
      i,
      x,
      y,
      c,
      xp,
      yp,
      done,
      k,
      pick_card,
      who_go,
      cur_card,
      cur_x,
      cur_y;
   char
      buffer[32];
   int
      num_battle_cards[2];
   CARD_INFO
      place_card,
      battle_deck1[5],
      battle_deck2[5];

   CursorOff();

   CreateBattleDeck(&player[0], battle_deck1);
   if(play_against_computer)
      CreateComputerBattleDeck(battle_deck2, battle_deck1);
   else
      CreateBattleDeck(&player[1], battle_deck2);

   num_battle_cards[0] = 5;
   num_battle_cards[1] = 5;
   who_go = rand() % 2;
   done = 0;
   pick_card = 1;
   cur_card = 0;


   while(!done)
   {
      /* Draw Player 1 cards */
      DrawBattleDeck(battle_deck1, num_battle_cards[0], 0, 1);
      /* Draw Player2/computer cards */
      DrawBattleDeck(battle_deck2, num_battle_cards[1], 65, 4);

      DrawField();

      if(pick_card)
      {
         switch(who_go)
         {
            case 0:
               DrawCard(0, (cur_card * 3), 7, &battle_deck1[cur_card]);
               break;
            case 1:
               DrawCard(65, (cur_card * 3), 7, &battle_deck2[cur_card]);
               break;
         }
      } else
      {
         /* draw where the card is being placed */
         i = 7;
         if(field_cards[cur_y][cur_x].name[0])
            i = 8;

         DrawCard(18 + (cur_x * 15), (cur_y * 8), i, &place_card);
         if(game_type == 1 || game_type == 3) /* element */
         {
            sprintf(buffer, "(%+d)", GetFieldElementBonus(&place_card,
               cur_x, cur_y));
            c = 14 - strlen(buffer) + (cur_x * 15) + 18;
            for(k = 0; buffer[k]; k++)
            {
               Cursor(c + k, 6 + (cur_y * 8));
               WriteChar(buffer[k], 15, i);
            }
         }
      }


      fflush(stdin);
      if(play_against_computer && who_go == 1) /* computer's turn */
         k = ComputerThink(pick_card, battle_deck2, num_battle_cards[1]);
      else
         k = dio_getchar();
      fflush(stdin);

      if(pick_card)
      {
         if(k == UP)
         {
            if(cur_card)
               cur_card--;
         } else if(k == DOWN)
         {
            if(cur_card < (num_battle_cards[who_go] - 1))
               cur_card++;
         } else if(k == ENTER)
         {
            pick_card = 0;
            cur_x = 0;
            cur_y = 0;

            dio_clear();
            switch(who_go)
            {
               case 0:
                  memcpy(&place_card, &battle_deck1[cur_card], sizeof(CARD_INFO));
                  if(cur_card != (num_battle_cards[0] - 1))
                  {
                     memcpy(&battle_deck1[cur_card], &battle_deck1[cur_card+1],
                        (num_battle_cards[0] - cur_card) * sizeof(CARD_INFO));
                  }
                  num_battle_cards[0]--;
                  break;
               case 1:
                  memcpy(&place_card, &battle_deck2[cur_card], sizeof(CARD_INFO));
                  if(cur_card != (num_battle_cards[1] - 1))
                  {
                     memcpy(&battle_deck2[cur_card], &battle_deck2[cur_card+1],
                        (num_battle_cards[1] - cur_card) * sizeof(CARD_INFO));
                  }
                  num_battle_cards[1]--;
                  break;
            }
         }
      } else  /* Placing a card on the field. */
      {
         if(k == UP)
         {
            if(cur_y)
               cur_y--;
         } else if(k == DOWN)
         {
            if(cur_y < 2)
               cur_y++;
         } else if(k == LEFT)
         {
            if(cur_x)
               cur_x--;
         } else if(k == RIGHT)
         {
            if(cur_x < 2)
               cur_x++;
         } else if(k == ENTER)
         {
            if(field_cards[cur_y][cur_x].name[0] == 0)
            {
               memcpy(&field_cards[cur_y][cur_x], &place_card, sizeof(CARD_INFO));
               field_color[cur_y][cur_x] = (who_go == 0) ? 1 : 4;
               SetupElementBonus(cur_x, cur_y);
               switch(game_type)
               {
                  case 0:
                  case 1:
                     NormalFlip((who_go == 0) ? 1 : 4, cur_x, cur_y);
                     break;
                  case 2:
                  case 3:
                     ChainFlip((who_go == 0) ? 1 : 4, cur_x, cur_y);
                     break;
               }

               done = CheckForWin();
               if(!done)
               {
                  who_go ^= 1;
                  pick_card = 1;
                  cur_card = 0;
               } else
               {
                  DrawField();
               }
            }
         } else if(k == ESCAPE)
         {
            pick_card = 1;
            cur_card = 0;
            switch(who_go)
            {
               case 0:
                  memcpy(&battle_deck1[num_battle_cards[0]], &place_card, sizeof(CARD_INFO));
                  num_battle_cards[0]++;
                  break;
               case 1:
                  memcpy(&battle_deck2[num_battle_cards[1]], &place_card, sizeof(CARD_INFO));
                  num_battle_cards[1]++;
                  break;
            }
         }
      }
   }

   if(!play_against_computer)
   {
      sprintf(buffer, "%s won! Press a key.", player[done-1].name);
   } else
   {
      if(done == 1)
         sprintf(buffer, "%s won! Press a key.", player[0].name);
      else
         sprintf(buffer, "The computer won! Press a key.");
   }

   MessageBox(buffer);

   if(done == 1)
   {
      player[0].wins++;
      player[0].game_points += win_points;

      if(!play_against_computer)
         player[1].losses++;
   } else
   {
      player[0].losses++;

      if(!play_against_computer)
      {
         player[1].wins++;
         player[1].game_points += win_points;
      }
   }


   SavePlayer(&player[0]);

   if(!play_against_computer)
      SavePlayer(&player[1]);

   CursorOn();
   dio_clear();
}