#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include <dos.h>
#include <dir.h>

#include "dio.h"
#include "cards.h"
#include "players.h"
#include "textgrph.h"


typedef struct tag_player_names
{
   char
      name[9];
   struct tag_player_names
      *prev,
      *next;
} PLAYER_NAMES;

PLAYER_NAMES
   *p_names,
   *p_names_tail;

int
   game_state,
   exit_game,
   num_player_names;
PLAYER_INFO
   player[2];


int AddPlayerName(char *name)
{
   PLAYER_NAMES
      *nn;

   nn = (PLAYER_NAMES *)calloc(1, sizeof(PLAYER_NAMES));
   if(!nn)
      return 0;

   strcpy(nn->name, name);
   nn->next = NULL;
   nn->prev = NULL;

   if(p_names == NULL)
   {
      p_names = nn;
      nn->prev = NULL;
   } else
   {
      p_names_tail->next = nn;
      nn->prev = p_names_tail;
   }
   p_names_tail = nn;
   num_player_names++;
   return 1;
}


void RemovePlayerFromList(char *name)
{
   PLAYER_NAMES
      *cur;
   int
      done = 0;

   cur = p_names;
   while(!done && cur)
   {
      if(!stricmp(cur->name, name))
      {
         done = 1;

         if(cur->prev)
            cur->prev->next = cur->next;
         else
            p_names = cur->next;

         if(cur->next)
            cur->next = cur->prev;
         else
            p_names_tail = cur->prev;

         free(cur);
      } else
      {
         cur = cur->next;
      }
   }
   num_player_names--;
}


void FreePlayerNames(void)
{
   PLAYER_NAMES
      *cur,
      *tmp;

   cur = p_names;
   while(cur)
   {
      tmp = cur->next;
      free(cur);
      cur = tmp;
   }

   p_names = NULL;
   p_names_tail = NULL;
}


/*
   New players are always created into a file.
*/
void CreateNewPlayer(void)
{
   PLAYER_NAMES
      *tmp,
      *cur;
   int
      i,
      exit_key,
      good_name;
   char
      *cp,
      buffer[9],
      fname[13];
   FILE
      *fp;
   PLAYER_INFO
      play;

   dio_clear();


   memset(buffer, 0, 9);
   good_name = 0;
   do
   {
      Grid(0, 1, 79, 24, 3, 1, 2);

      Cursor(0, 0);
      RepeatChar(' ', 80, 14, 1);
      WriteString("Create New Player", 14, 1, 32, 0);

      Grid(12, 10, 72, 14, 0, 0, 0);
      Box(10, 9, 70, 13, SINGLE_BOX, 1, 14, 1);

      WriteString("Enter you name. It can only be 8 letters long.", 14, 1, 12, 11);
      WriteString("Name:", 14, 1, 12, 12);

      dio_textcolor(14);
      dio_textbackground(1);

      dio_padchar('_');
      exit_key = dio_edit(buffer, 12, 18, 9, 8);
      dio_padchar(' ');
      fflush(stdin);

      if(exit_key == ESCAPE)
      {
         CursorOff();
         MessageBox("Character creaation aborted.");
         CursorOn();
         dio_clear();
         dio_textcolor(7);
         dio_textbackground(0);
         return;
      }

      if(buffer[0] == 0)
      {
         CursorOff();
         MessageBox("Your name must contain at least 1 letter.");
         CursorOn();
         continue;
      }

      if(strlen(buffer) <= 8)
      {
         cp = strchr(buffer, ' ');
         if(cp)
         {
            CursorOff();
            MessageBox("Your name cannot contain spaces. Try Again.");
            CursorOn();
            continue;
         }

         cur = p_names;
         good_name = 1;
         while(good_name && cur)
         {
            tmp = cur->next;
            if(!stricmp(cur->name, buffer))
            {
               CursorOff();
               MessageBox("That name is taken already. Try Again.");
               CursorOn();
               good_name = 0;
            }
            cur = tmp;
         }
      }
   } while(!good_name);

   sprintf(fname, "%s.crd", buffer);
   fp = fopen(fname, "wb");
   if(!fp)
   {
      CursorOff();
      MessageBox("There was an error creating your character."
                 " Please try again.");
      CursorOn();
      dio_clear();
      dio_textcolor(7);
      dio_textbackground(0);
      return;
   }

   memset(&play, 0, sizeof(PLAYER_INFO));

   buffer[0] = toupper(buffer[0]);
   for(i = 1; buffer[i]; i++);
      buffer[i] = tolower(buffer[i]);

   strcpy(play.name, buffer);
   play.game_points = 0L;
   play.number_of_cards = 8;

   CreateStartingDeck(play.deck);
   play.wins = 0;
   play.losses = 0;

   fwrite(play.name, 1, 9, fp);
   fwrite(&play.game_points, 4, 1, fp);
   fwrite(&play.number_of_cards, 2, 1, fp);
   fwrite(play.deck, 4, 500, fp);
   fwrite(&play.wins, 2, 1, fp);
   fwrite(&play.losses, 2, 1, fp);

   fflush(fp);
   fclose(fp);

   if(!AddPlayerName(play.name))
   {
      CursorOff();
      MessageBox("There was a problem adding character name to memory!");
      CursorOn();
   } else
   {
      CursorOff();
      MessageBox("Character created!");
      CursorOn();
   }

   dio_clear();
   dio_textcolor(7);
   dio_textbackground(0);
}


void LoadPlayerNames(void)
{
   char
      *cp;
   int
      i;
   struct find_t
      finfo;

   atexit(FreePlayerNames);
   if(!_dos_findfirst("*.crd", FA_ARCH, &finfo))
   {
      cp = strchr(finfo.name, '.');
      if(cp)
         *cp = 0;

      finfo.name[0] = toupper(finfo.name[0]);
      for(i = 1; finfo.name[i]; i++)
         finfo.name[i] = tolower(finfo.name[i]);

      AddPlayerName(finfo.name);

      while(!_dos_findnext(&finfo))
      {
         cp = strchr(finfo.name, '.');
         if(cp)
            *cp = 0;

         finfo.name[0] = toupper(finfo.name[0]);
         for(i = 1; finfo.name[i]; i++)
            finfo.name[i] = tolower(finfo.name[i]);

         AddPlayerName(finfo.name);
      }
   }

   if(num_player_names)
      printf("%d Player Names loaded.\n", num_player_names);
   else
      printf("No player files exist.\n");
}



int LoadPlayer(PLAYER_INFO *p, char *name)
{
   char
      buffer[13];
   FILE
      *fp;

   sprintf(buffer, "%s.crd", name);
   fp = fopen(buffer, "rb");
   if(!fp)
      return 0;

   fread(p->name, 1, 9, fp);
   fread(&p->game_points, 4, 1, fp);
   fread(&p->number_of_cards, 2, 1, fp);
   fread(p->deck, 4, 500, fp);
   fread(&p->wins, 2, 1, fp);
   fread(&p->losses, 2, 1, fp);

   fclose(fp);
   return 1;
}


int SavePlayer(PLAYER_INFO *p)
{
   char
      buffer[13];
   FILE
      *fp;

   sprintf(buffer, "%s.crd", p->name);
   fp = fopen(buffer, "wb");
   if(!fp)
      return 0;

   fwrite(p->name, 1, 9, fp);
   fwrite(&p->game_points, 4, 1, fp);
   fwrite(&p->number_of_cards, 2, 1, fp);
   fwrite(p->deck, 4, 500, fp);
   fwrite(&p->wins, 2, 1, fp);
   fwrite(&p->losses, 2, 1, fp);

   fflush(fp);
   fclose(fp);
   return 1;
}


/*
   Selects one of the players to use.
   Returns 1 if a character was selected or 0 if not.
*/
int SelectPlayer(char *cant_be, PLAYER_INFO *p)
{
   int
      i,
      line,
      column,
      done,
      name_offset,
      redraw,
      cur_spot,
      exit_key;
   char
      the_name[9],
      buffer[16];
   PLAYER_NAMES
      *tmp,
      *cur;

   dio_clear();
   CursorOff();

   done = 0;
   cur_spot = 0;
   cur = p_names;
   redraw = 1;
   name_offset = 0;
   while(!done)
   {
      /* Display all character names */
      if(redraw)
      {
         Grid(0, 1, 79, 24, 3, 1, 2);

         Cursor(0, 0);
         RepeatChar(' ', 80, 14, 1);
         WriteString("Select Player", 14, 1, 33, 0);

         Grid(35, 3, 49, 23, 0, 0, 0);
         Box(33, 2, 47, 22, SINGLE_BOX, 1, 14, 1);

         for(tmp = cur, i = 0; i < 19 && tmp; i++)
         {
            sprintf(buffer, " %8s  ", tmp->name);
            tmp = tmp->next;

            if(i == cur_spot)
               WriteString(buffer, 0, 3, 35, 3 + i);
            else
               WriteString(buffer, 14, 1, 35, 3 + i);
         }
         redraw = 0;
      }

      fflush(stdin);
      exit_key = dio_getchar();
      fflush(stdin);

      if(exit_key == ESCAPE)
      {
         dio_clear();
         CursorOn();
         return 0;
      } else if(exit_key == DOWN)
      {
         if(cur_spot == 18)
         {
            if(name_offset < (num_player_names - 19))
            {
               name_offset++;
               cur = cur->next;
               redraw = 1;
            }
         } else
         {
            if(cur_spot < (num_player_names - 1))
            {
               cur_spot++;
               redraw = 1;
            }
         }
      } else if(exit_key == UP)
      {
         if(cur_spot)
         {
            cur_spot--;
            redraw = 1;
         } else
         {
            if(name_offset)
            {
               name_offset--;
               cur = cur->prev;
               redraw = 1;
            }
         }
      } else if(exit_key == ENTER)
      {
         /* See if it is a valid name */
         for(tmp = cur, i = 0; i < cur_spot; i++)
            tmp = tmp->next;

         if(cant_be && !stricmp(cant_be, tmp->name))
         {
            MessageBox("That person is already loaded.");
            redraw = 1;
         } else
         {
            strcpy(the_name, tmp->name);
            done = 1;
         }
      }
   }

   if(!LoadPlayer(p, the_name))
   {
      MessageBox("There was an error loading the character.");
      dio_clear();
      CursorOn();
      return 0;
   }
   CursorOn();
   return 1;
}


void RemovePlayer(void)
{
   int
      i,
      k,
      line,
      column,
      done,
      name_offset,
      redraw,
      cur_spot,
      exit_key;
   char
      fname[13],
      buffer[16];
   PLAYER_NAMES
      *tmp,
      *cur;

   dio_clear();
   CursorOff();

   done = 0;
   cur_spot = 0;
   cur = p_names;
   redraw = 1;
   name_offset = 0;
   while(!done)
   {
      /* Display all character names */
      if(redraw)
      {
         Grid(0, 1, 79, 24, 3, 1, 2);

         Cursor(0, 0);
         RepeatChar(' ', 80, 14, 1);
         WriteString("Remove Player", 14, 1, 33, 0);

         Grid(35, 3, 49, 23, 0, 0, 0);
         Box(33, 2, 47, 22, SINGLE_BOX, 1, 14, 1);

         for(tmp = cur, i = 0; i < 19 && tmp; i++)
         {
            sprintf(buffer, " %8s  ", tmp->name);
            tmp = tmp->next;

            if(i == cur_spot)
               WriteString(buffer, 0, 3, 35, 3 + i);
            else
               WriteString(buffer, 14, 1, 35, 3 + i);
         }
         redraw = 0;
      }

      fflush(stdin);
      exit_key = dio_getchar();
      fflush(stdin);

      if(exit_key == ESCAPE)
      {
         dio_clear();
         CursorOn();
         return;
      } else if(exit_key == DOWN)
      {
         if(cur_spot == 18)
         {
            if(name_offset < (num_player_names - 19))
            {
               name_offset++;
               cur = cur->next;
               redraw = 1;
            }
         } else
         {
            if(cur_spot < (num_player_names - 1))
            {
               cur_spot++;
               redraw = 1;
            }
         }
      } else if(exit_key == UP)
      {
         if(cur_spot)
         {
            cur_spot--;
            redraw = 1;
         } else
         {
            if(name_offset)
            {
               name_offset--;
               cur = cur->prev;
               redraw = 1;
            }
         }
      } else if(exit_key == ENTER)
      {
         /* See if it is a valid name */
         for(tmp = cur, i = 0; i < cur_spot; i++)
            tmp = tmp->next;

         WriteString("Are you sure? (y/n)", 14, 1, 2, 24);

         do
         {
            fflush(stdin);
            k = dio_getchar();
            fflush(stdin);
         } while(k != 'y' && k != 'Y' && k != 'n' && k != 'N');

         if(k == 'y' || k == 'Y')
         {
            sprintf(fname, "%s.crd", tmp->name);
            remove(fname);

            RemovePlayerFromList(tmp->name);
            MessageBox("Character removed.");
            done = 1;
         }

      }
   }

   dio_clear();
   CursorOn();
}


void PlayerMenu(void)
{
   static char
      *player_menu[] = {
         "Create New Player",
         "Select Player",
         "Remove Player",
         "Quit Game"
      };
   int
      done,
      choice;

   done = 0;
   while(!done)
   {
      choice = Menu("PLAYER MENU", player_menu, 4);

      switch(choice)
      {
         case 0:
            CreateNewPlayer();
            break;
         case 1:
            if(num_player_names > 0 && SelectPlayer(NULL, &player[0]))
            {
               dio_clear();
               game_state = STATE_GAME_MENU;
               done = 1;
            }
            break;
         case 2:
            if(num_player_names > 0)
               RemovePlayer();
            break;
         case 3:
            dio_clear();
            exit_game = 1;
            done = 1;
            break;
      }
   }
}


void GiveCard(PLAYER_INFO *giver, PLAYER_INFO *taker)
{
   int
      done,
      cur_card,
      redraw,
      k;
   unsigned short
      id,
      level_num;
   long
      cost;
   char
      buffer[80];

   CursorOff();

   if(giver->number_of_cards == 0)
   {
      dio_clear();
      MessageBox("Sorry you don't have any cards to give.");
      dio_clear();
      CursorOn();
      return;
   } else if(taker->number_of_cards == 500)
   {
      dio_clear();
      sprintf(buffer, "Sorry %s can't have anymore cards.", taker->name);
      MessageBox(buffer);
      dio_clear();
      CursorOn();
      return;
   }

   sprintf(buffer, "%s Give Card to %s", giver->name, taker->name);
   DisplayCardInfoPageBackground(buffer, giver);

   cur_card = 0;
   done = 0;
   redraw = 1;
   while(!done)
   {
      if(redraw)
      {
         DisplayCardInfoPage();
         sprintf(buffer, "Card #%d of %d", cur_card + 1,
            giver->number_of_cards);
         WriteString(buffer, 14, 1, 3, 5);

         level_num = (unsigned short)(giver->deck[cur_card] >> 16L) - 1;
          id = (unsigned short)(giver->deck[cur_card] & 0xFFFF);

         DisplayCardData(level_num, id);
         redraw = 0;
      }

      fflush(stdin);
      k = dio_getchar();
      fflush(stdin);

      if(k == ESCAPE)
      {
         done = 1;
      } else if(k == UP)
      {
         if(cur_card)
         {
            redraw = 1;
            cur_card--;
         }
      } else if(k == DOWN)
      {
         if(cur_card < (giver->number_of_cards - 1))
         {
            redraw = 1;
            cur_card++;
         }
      } else if(k == ENTER)
      {
         WriteString("Are you sure? (y/n)", 14, 1, 3, 12);

         do
         {
            fflush(stdin);
            k = dio_getchar();
            fflush(stdin);
         } while(k != 'y' && k != 'Y' && k != 'n' && k != 'N');

         redraw = 1;
         if(k == 'y' || k == 'Y')
         {
            MessageBox("Card Given!");

            sprintf(buffer, "%s Give Card to %s", giver->name, taker->name);
            DisplayCardInfoPageBackground(buffer, giver);

            if(giver->number_of_cards == 1)
            {
               memset(giver->deck, 0, 4 * 500);
               done = 1;
            } else
            {
               if(cur_card != (giver->number_of_cards - 1))
               {
                  memcpy(&giver->deck[cur_card], &giver->deck[cur_card+1],
                     (giver->number_of_cards - cur_card) * 4);
               } else
               {
                  cur_card--;
               }
            }

            giver->number_of_cards--;

            if(giver->number_of_cards == 0)
               done = 1;

            taker->deck[taker->number_of_cards] =
               (id + ((long)(level_num + 1) << 16L));
            taker->number_of_cards++;
         }
      }
   }

   CursorOn();
   dio_clear();
   SavePlayer(giver);
   SavePlayer(taker);
}


void GiveGamePoints(PLAYER_INFO *giver, PLAYER_INFO *taker)
{
   long
      number;
   char
      buffer[80];
   int
      k;


   Grid(0, 1, 79, 24, 3, 1, 2);

   Cursor(0, 0);
   RepeatChar(' ', 80, 14, 1);
   WriteString("Give Game Points", 14, 1, 32, 0);

   Grid(12, 9, 72, 14, 0, 0, 0);
   Box(10, 8, 70, 13, SINGLE_BOX, 1, 14, 1);

   sprintf(buffer, "How many points will %s give to %s?", giver->name,
      taker->name);
   WriteString(buffer, 14, 1, 12, 9);

   sprintf(buffer, "%s has %ld game points.", giver->name,
      giver->game_points);
   WriteString(buffer, 14, 1, 12, 10);

   WriteString("Amount:", 14, 1, 12, 12);

   dio_textcolor(14);
   dio_textbackground(1);

   memset(buffer, 0, 80);
   fflush(stdin);
   dio_padchar('_');
   fflush(stdin);
   k = dio_edit(buffer, 12, 20, 10, 11);
   fflush(stdin);
   dio_padchar(' ');

   dio_textcolor(7);
   dio_textbackground(0);

   if(k == ESCAPE || strlen(buffer) == 0)
   {
      CursorOff();
      MessageBox("Canceling giving points.");
      CursorOn();
      dio_clear();
      return;
   }

   number = atol(buffer);

   if(number <= 0 || number > giver->game_points)
   {
      CursorOff();
      if(number == 0)
         MessageBox("You can't give 0 points.");
      else if (number < 0)
         MessageBox("You can't give negative points.");
      else
         MessageBox("You don't have that many points.");

      CursorOn();
      dio_clear();
      return;
   }

   giver->game_points -= number;
   taker->game_points += number;

   CursorOff();
   MessageBox("Points given!");
   CursorOn();
   dio_clear();
   SavePlayer(giver);
   SavePlayer(taker);
}


void SameComputerTrade(void)
{
   int
      done,
      k;
   static char
      *in_trade[] = {
         "XXXXXXXX Give Card To XXXXXXXX Extra Space",
         "XXXXXXXX Give Card To XXXXXXXX Extra Space",
         "XXXXXXXX Give Points To XXXXXXXX Extra Space",
         "XXXXXXXX Give Points To XXXXXXXX Extra Space",
         "View XXXXXXXX's Cards Extra Space",
         "View XXXXXXXX's Cards Extra Space",
         "Go Back"
      };
   char
      buffer[80];

   if(!SelectPlayer(player[0].name, &player[1]))
      return;

   sprintf(in_trade[0], "%s Give Card to %s", player[0].name, player[1].name);
   sprintf(in_trade[1], "%s Give Card to %s", player[1].name, player[0].name);
   sprintf(in_trade[2], "%s Give Points to %s", player[0].name, player[1].name);
   sprintf(in_trade[3], "%s Give Points to %s", player[1].name, player[0].name);
   sprintf(in_trade[4], "View %s's Cards", player[0].name);
   sprintf(in_trade[5], "View %s's Cards", player[1].name);

   done = 0;
   while(!done)
   {
      k = Menu("TRADE MENU", in_trade, 7);

      switch(k)
      {
         case 0:
            GiveCard(&player[0], &player[1]);
            break;
         case 1:
            GiveCard(&player[1], &player[0]);
            break;
         case 2:
            GiveGamePoints(&player[0], &player[1]);
            break;
         case 3:
            GiveGamePoints(&player[1], &player[0]);
            break;
         case 4:
            ViewPlayersCards(&player[0]);
            break;
         case 5:
            ViewPlayersCards(&player[1]);
            break;
         case 6:
            done = 1;
      }
   }
   dio_clear();
}



void TradeCards(void)
{
   static char
      *trade_menu1[] = {
         "Same Computer",
         "Serial Link",
         "IPX Network",
         "Go Back"
      };
   int
      done,
      k;

   done = 0;
   while(!done)
   {
      k = Menu("TRADE MENU", trade_menu1, 4);

      switch(k)
      {
         case 0:
            SameComputerTrade();
            break;
         case 1:
            break;
         case 2:
            break;
         case 3:
            done = 1;
            break;
      }
   }
   dio_clear();
}
