#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

#include "textgrph.h"

/*
   Places the cursor at a specific x,y position.
*/
void Cursor(int x, int y)
{
   union REGS
      regs;

   regs.h.ah = 0x02;
   regs.h.bh = 0;
   regs.h.dh = y;
   regs.h.dl = x;

   int86(0x10, &regs, &regs);
}


/*
   Sets the cursors size
*/
void SetCursorSize(int start, int end)
{
   union REGS
      regs;

   regs.h.ah = 0x01;
   regs.h.ch = start;
   regs.h.cl = end;

   int86(0x10, &regs, &regs);
}


/*
   Gets information on current cursor position
*/
void GetCursor(int *x, int *y, int *page, int *start, int *end)
{
   union REGS
      regs;

   regs.h.ah = 0x03;
   regs.h.bh = 0;

   int86(0x10, &regs, &regs);

   *y = (int)regs.h.dh;
   *x = (int)regs.h.dl;
   *page = (int)regs.h.bh;
   *start = (int)regs.h.ch;
   *end = (int)regs.h.cl;
}


/*
   Gets the current video mode
*/
void GetVideo(int *columns, int *displayMode, int *displayPage)
{
   union REGS
      regs;

   regs.h.ah = 0x0F;
   int86(0x10, &regs, &regs);
  
   *columns = (int)regs.h.ah;
   *displayMode = (int)regs.h.al;
   *displayPage = (int)regs.h.bh;
}


/*
   Sets the video mode
*/
void SetVideo(int displayMode)
{
   union REGS
      regs;

   regs.h.ah = 0x00;
   regs.h.al = displayMode;

   int86(0x10, &regs, &regs);
}


/*
   Sets the border color
*/
void SetBorderColor(int color)
{
   union REGS
      regs;

   regs.h.ah = 0x0B;
   regs.h.bh = 0;
   regs.h.bl = color;

   int86(0x10, &regs, &regs);
}


/*
   Writes a character to screen
*/
void WriteChar(char ch, int fcolor, int bcolor)
{
   union REGS
      regs;

   regs.h.ah = 0x09;
   regs.h.al = ch;
   regs.h.bh = 0;
   regs.h.bl = (bcolor << 4) | fcolor;
   regs.x.cx = 1;

   int86(0x10, &regs, &regs);
}


/*
   Repeats a character starting at cursor location
*/

void RepeatChar(char ch, int howMany, int fcolor, int bcolor)
{
   union REGS
      regs;

   regs.h.ah = 0x09;
   regs.h.al = ch;
   regs.h.bh = 0;
   regs.h.bl = (bcolor << 4) | fcolor;
   regs.x.cx = howMany;

   int86(0x10, &regs, &regs);
}


/*
   Writes a string to screen
*/
void OldWriteString(char *string, int fcolor, int bcolor, int x, int y)
{
   int
      i;

   for(i = 0; string[i]; i++)
   {
      Cursor(x + i, y);
      WriteChar(string[i], fcolor, bcolor);
   }
}


/*
   Writes a string to screen
*/
void WriteString(char *string, int fcolor, int bcolor, int x, int y)
{
   int
      i;

   for(i = 0; string[i] && x < 80; i++)
   {
      if(x >= 0)
      {
         Cursor(x, y);
         WriteChar(string[i], fcolor, bcolor);
      }
      x++;
   }
}


void DisplayTextLine(char *string, int fc, int bc, int x, int y, int pos)
{
   int
      i;

   if(strlen(string) < pos)
      return;

   for(i = 0; string[i+pos] && x < 79; i++)
   {
      if(x >= 0)
      {
         Cursor(x, y);
         WriteChar(string[i+pos], fc, bc);
      }
      x++;
   }
}

/*
   Draws a box on screen
*/
void Box(int x0, int y0, int x1, int y1, int type,
         int fill, int fcolor, int bcolor)
{
   int
      row;
   static unsigned char
      DBL_BOX[6] = {201, 205, 187, 186, 200, 188},
      SGL_BOX[6] = {218, 196, 191, 179, 192, 217},
      BLK_BOX[6] = {0, 0, 0, 0, 0, 0},
      *USE_BOX;
  
   switch (type)
   {
      case DOUBLE_BOX:
      {
         USE_BOX = DBL_BOX;
         break;
      }
      case SINGLE_BOX:
      {
         USE_BOX = SGL_BOX;
         break;
      }
      case BLANK_BOX:
      {
         USE_BOX = BLK_BOX;
         break;
      }
      default:
      {
         USE_BOX = DBL_BOX;
      }
   }
  
   Cursor(x0, y0);
   WriteChar(USE_BOX[0], fcolor, bcolor);
   Cursor(x1, y0);
   WriteChar(USE_BOX[2], fcolor, bcolor);
  
   Cursor(x0 + 1, y0);
   RepeatChar(USE_BOX[1], x1 - x0 - 1, fcolor, bcolor);
  
   Cursor(x0 + 1, y1);
   RepeatChar(USE_BOX[1], x1 - x0 - 1, fcolor, bcolor);
  
   for(row = y0 + 1; row < y1; row++)
   {
      Cursor(x0, row);
      WriteChar(USE_BOX[3], fcolor, bcolor);
      Cursor(x1, row);
      WriteChar(USE_BOX[3], fcolor, bcolor);
   }
  
   Cursor(x0, y1);
   WriteChar(USE_BOX[4], fcolor, bcolor);
   Cursor(x1, y1);
   WriteChar(USE_BOX[5], fcolor, bcolor);
  
   if(fill == 1)
   {
      for(row = y0 + 1; row < y1; row++)
      {
         Cursor(x0 + 1, row);
         RepeatChar(' ', ((x1 - x0) - 1), fcolor, bcolor);
      }
   }
}


/*
   Scrolls a part of the screen
*/
void Scroll(int x, int y, int width, int height,
            int nbr, int direction)
{
   union REGS
      regs;

   regs.h.ah = direction;
   regs.h.al = nbr;
   regs.h.ch = y;
   regs.h.cl = x;
   regs.h.dh = y + height;
   regs.h.dl = x + width;
   regs.h.bh = 0;

   int86(0x10, &regs, &regs);
}


/*
   Hides the cursor
*/
void CursorOff(void)
{
   union REGS
      regs;

   regs.h.ah = 0x01;
   regs.x.cx = 0x0F00;

   int86(0x10, &regs, &regs);
}


/*
   Displays the Cursor
*/
void CursorOn(void)
{
   union REGS
      regs;

   regs.h.ah = 0x01;
   regs.x.cx = 0x0607;

   int86(0x10, &regs, &regs);
}


/*
   Clears the screen in the colors fcolor and bcolor
*/
void ClearScreen(int fcolor, int bcolor)
{
   union REGS
      regs;

   regs.h.ah = SCROLL_UP;
   regs.h.al = 0;
   regs.h.ch = 0;
   regs.h.cl = 0;
   regs.h.dh = 24;
   regs.h.dl = 79;
   regs.h.bh = (bcolor << 4) | fcolor;

   int86(0x10, &regs, &regs);
   Cursor(0, 0);
}


/*
   Displays a grid background
*/
void Grid(int x0, int y0, int x1, int y1, int fcolor,
          int bcolor, int type)
{
   int
      ch,
      row;

   switch(type)
   {
      case 0:
      {
         ch = ' ';
         break;
      }
      case 1:
      {
         ch = 176;
         break;
      }
      case 2:
      {
         ch = 177;
         break;
      }
      case 3:
      {
         ch = 178;
      }
   }
  
   for(row = y0; row < (y1 + 1); row++)
   {
      Cursor(x0, row);
      RepeatChar((char)ch, x1 - x0 + 1, fcolor, bcolor);
   }
}


/*
    Saves a portion of the screen
*/
char *SaveScreenArea(int x0, int y0, int x1, int y1)
{
   char
      *screenBuffer;
   union REGS
      regs;
   int totalSpace,
      row = y0,
      col,
      ctr,
      trow,
      tcol,
      page,
      curSt,
      curEnd;
  
   GetCursor(&tcol, &trow, &page, &curSt, &curEnd);
  
   totalSpace = (((y1 - y0 + 1) * (x1 - x0 + 1)) * 2) + 5;
  
   screenBuffer = (char *)malloc(totalSpace);
   if(!screenBuffer)
   {
      printf("\nUnable to allocate %d bytes.\n");
      return NULL;
   }

   *(screenBuffer + 0) = (char)x0;
   *(screenBuffer + 1) = (char)y0;
   *(screenBuffer + 2) = (char)x1;
   *(screenBuffer + 3) = (char)y1;
   *(screenBuffer + 4) = (char)page;
  
   ctr = 5;

   while (row <= y1)
   {
      col = x0;
      while (col <= x1)
      {
         Cursor(col, row);
      
         regs.h.ah = 0x08;
         regs.h.bh = page;

         int86(0x10, &regs, &regs);

         *(screenBuffer + ctr++) = (char)regs.h.al;
         *(screenBuffer + ctr++) = (char)regs.h.ah;
      
         col++;
      }
      row++;
   }
  
  return screenBuffer;
}


/*
   Restores a portion of the screen
*/
void RestoreScreenArea(char *buffer)
{
   union REGS
      regs;
   int
      x0,
      x1,
      y0,
      y1,
      page,
      ctr = 5,
      row,
      col;
  
   x0 = (int)*(buffer + 0);
   y0 = (int)*(buffer + 1);
   x1 = (int)*(buffer + 2);
   y1 = (int)*(buffer + 3);
   page = (int)*(buffer + 4);
  
   row = y0;
  
   while (row <= y1)
   {
      col = x0;
      while (col <= x1)
      {
         Cursor(col, row);
      
         regs.h.ah = 0x09;
         regs.h.bh = page;
         regs.h.al = *(buffer + ctr++);
         regs.h.bl = *(buffer + ctr++);
         regs.x.cx = 1;

         int86(0x10, &regs, &regs);

         col++;
      }
      row++;
   }

  free(buffer);
}

