#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>

#include "pakfile.h"


#define MAX_PAK_FILES      30
#define MAX_OPEN_FILES     30

typedef struct
{
   char
      name[128];
   long
      offset,
      length;
} PAK_FILE_ENTRY;

typedef struct
{
   FILE
      *fp;
   long
      num_entries;
   PAK_FILE_ENTRY
      *entry;
} PAK_FILE;


typedef struct
{
   FILE
      *fp;
   long
      group_file,
      offset,
      start,
      length;
} OPEN_PAK_FILE;

long
   num_pakfiles;
PAK_FILE
   pakfile[MAX_PAK_FILES];
OPEN_PAK_FILE
   openfiles[MAX_OPEN_FILES];



int FileExist(char *name)
{
   FILE
      *fp;

   fp = fopen(name, "rb");
   if(!fp)
      return 0;
   fclose(fp);
   return 1;
}


void InitPakFiles(void)
{
   int
      i;

   memset(pakfile, 0, sizeof(PAK_FILE) * MAX_PAK_FILES);
   memset(openfiles, 0, sizeof(OPEN_PAK_FILE) * MAX_OPEN_FILES);
   atexit(CloseAllPakFiles);
}


/*
   Returns 1 if the pakfile was added to the list.
   Otherwise returns 0.
*/
int AddPakFile(char *fname)
{
   FILE
      *fp;
   long
      number,
      cnt,
      index;

   /* Look for an empty slot. */
   for(index = 0; index < MAX_PAK_FILES; index++)
   {
      if(pakfile[index].fp == NULL)
         break;
   }

   if(index >= MAX_PAK_FILES)
      return 0;

   fp = fopen(fname, "rb");
   if(!fp)
      return 0;


   fread(&number, sizeof(long), 1, fp);
   pakfile[index].entry = (PAK_FILE_ENTRY *)calloc(number, sizeof(PAK_FILE_ENTRY));
   if(!pakfile[index].entry)
   {
      fclose(fp);
      return 0;
   }

   for(cnt = 0; cnt < number; cnt++)
   {
      fread(pakfile[index].entry[cnt].name, sizeof(char), 128, fp);
      fread(&pakfile[index].entry[cnt].offset, sizeof(long), 1, fp);
      fread(&pakfile[index].entry[cnt].length, sizeof(long), 1, fp);
   }

   pakfile[index].num_entries = number;
   pakfile[index].fp = fp;
   return 1;
}


void CloseAllPakFiles(void)
{
   int
      i;

   for(i = 0; i < MAX_OPEN_FILES; i++)
      Pak_fclose(i);

   for(i = 0; i < MAX_PAK_FILES; i++)
   {
      if(pakfile[i].fp)
      {
         fclose(pakfile[i].fp);
         pakfile[i].fp = NULL;
         if(pakfile[i].entry)
         {
            free(pakfile[i].entry);
            pakfile[i].entry = NULL;
         }
      }
   }
}


/*
   If the entry cannot be found 0 will be returned.
   If the file is found it will be opened (if an individual file)
   and a openfile[] entry will be made as well as 1 being returned.
*/
int FindPakEntry(char *name, int index)
{
   FILE
      *fp;
   int
      i,
      j;

   if(FileExist(name)) /* Use the single file */
   {
      fp = fopen(name, "rb");
      if(!fp)
         return 0;

      openfiles[index].fp = fp;
      openfiles[index].group_file = 0;
      openfiles[index].offset = 0;
      openfiles[index].start = 0;
      openfiles[index].length = filelength(fileno(fp));
      return 1;
   } else
   {
      for(i = 0; i < MAX_PAK_FILES; i++)
      {
         for(j = 0; j < pakfile[i].num_entries; j++)
         {
            if(!stricmp(pakfile[i].entry[j].name, name))
            {
               openfiles[index].fp = pakfile[i].fp;
               openfiles[index].group_file = 1;
               openfiles[index].offset = 0;
               openfiles[index].start = pakfile[i].entry[j].offset;
               openfiles[index].length = pakfile[i].entry[j].length;
               Pak_fseek(index, 0, SEEK_SET);
               return 1;
            }
         }
      }
   }
   return 0;
}


/*
   Returns -1 when the file can't be opened.
   Otherwise the index in the openfile[] is returned.
*/
int Pak_fopen(char *name)
{
   int
      index;

   for(index = 0; index < MAX_OPEN_FILES; index++)
   {
      if(openfiles[index].fp == NULL)
         break;
   }

   if(index >= MAX_OPEN_FILES)
      return -1;

   if(!FindPakEntry(name, index))
      return -1;
   return index;
}


void Pak_fclose(int fileno)
{
   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return;
   }

   if(!openfiles[fileno].group_file)
   {
      fclose(openfiles[fileno].fp);
   }
   openfiles[fileno].fp = NULL;
   openfiles[fileno].group_file = 0;
   openfiles[fileno].offset = 0;
   openfiles[fileno].start = 0;
   openfiles[fileno].length = 0;
}


/*
   Works like fread()
*/
unsigned Pak_fread(void *p, int size, unsigned number, int fileno)
{
   long
      num,
      v;

   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return 0;
   }

   if(ftell(openfiles[fileno].fp) !=
       (openfiles[fileno].offset + openfiles[fileno].start))
   {
      Pak_fseek(fileno, openfiles[fileno].offset, SEEK_SET);
   }

   v = size * number + openfiles[fileno].offset;
   if(v > openfiles[fileno].length) /* Trying to read too much */
   {
      openfiles[fileno].offset = openfiles[fileno].length;
      return 0;
   }

   num = fread(p, size, number, openfiles[fileno].fp);
   openfiles[fileno].offset += num * size;
   return num;
}


/*
   Works like fgetc()
*/
int Pak_fgetc(int fileno)
{
   int
      c;

   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return EOF;
   }

   if(ftell(openfiles[fileno].fp) !=
       (openfiles[fileno].offset + openfiles[fileno].start))
   {
      Pak_fseek(fileno, openfiles[fileno].offset, SEEK_SET);
   }

   if(openfiles[fileno].offset == openfiles[fileno].length)
      return EOF;

   c = fgetc(openfiles[fileno].fp);
   openfiles[fileno].offset++;
   return c;
}


/*
   Works like fgets()
*/
char *Pak_fgets(char *buffer, unsigned max_length, int fileno)
{
   char
      c;
   long
      done,
      cnt;

   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return 0;
   }

   if(ftell(openfiles[fileno].fp) !=
       (openfiles[fileno].offset + openfiles[fileno].start))
   {
      Pak_fseek(fileno, openfiles[fileno].offset, SEEK_SET);
   }

   for(done = 0, cnt = 0; !done && cnt < max_length;)
   {
      c = fgetc(openfiles[fileno].fp);
      if(c != EOF || openfiles[fileno].offset < openfiles[fileno].length)
         openfiles[fileno].offset++;
      if(c == 13)
      {
         fgetc(openfiles[fileno].fp); /* skip '\n' */
         openfiles[fileno].offset++;
         buffer[cnt++] = '\n';
         done = 1;
      } else if(c == EOF || openfiles[fileno].offset >= openfiles[fileno].length)
      {
         done = 1;
         if(cnt == 0)
            return NULL;
      } else
      {
         buffer[cnt++] = c;
      }
   }

   buffer[cnt] = '\0';

   return buffer;
}


/*
   Works like fseek.
*/
long Pak_fseek(int fileno, long offset, int mode)
{
   int
      pos = 0,
      val;

   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return 0;
   }

   switch(mode)
   {
      case SEEK_SET:
         pos = offset;
         break;
      case SEEK_CUR:
         pos = openfiles[fileno].offset + offset;
         break;
      case SEEK_END:
         pos = openfiles[fileno].length + offset;
         break;
      default:
         return 1;
   }

   if(pos < 0)
      pos = 0;
   if(pos > openfiles[fileno].length)
      pos = openfiles[fileno].length;

   openfiles[fileno].offset = pos;
   pos += openfiles[fileno].start;
   val = fseek(openfiles[fileno].fp, pos, SEEK_SET);

   return val;
}


long Pak_filelength(int fileno)
{
   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return 0;
   }

   return openfiles[fileno].length;
}


int Pak_feof(int fileno)
{
   if(fileno < 0 || fileno >= MAX_OPEN_FILES ||
      openfiles[fileno].fp == NULL)
   {
      return 1;
   }
   if(openfiles[fileno].group_file)
   {
      if(openfiles[fileno].offset == openfiles[fileno].length)
         return 1;
      else
         return 0;
   }
   return feof(openfiles[fileno].fp);
}
